@@ -17,9 +17,17 @@ import unittest
 import traceback
 from contextlib import contextmanager
 import logging
+from unittest import SkipTest
+
+import os
+
 import test.functional as tf
 from test.functional.s3api.s3_test_client import (
     Connection, get_boto3_conn, tear_down_s3)
+try:
+    import boto
+except ImportError:
+    boto = None
 
 
 def setUpModule():
@@ -44,8 +52,12 @@ class S3ApiBase(unittest.TestCase):
             logging.getLogger('boto').setLevel(logging.DEBUG)
 
     def setUp(self):
+        if not tf.config.get('s3_access_key'):
+            raise SkipTest('no s3api user configured')
         if 's3api' not in tf.cluster_info:
-            raise tf.SkipTest('s3api middleware is not enabled')
+            raise SkipTest('s3api middleware is not enabled')
+        if boto is None:
+            raise SkipTest('boto 2.x library is not installed')
         if tf.config.get('account'):
             user_id = '%s:%s' % (tf.config['account'], tf.config['username'])
         else:
@@ -81,8 +93,10 @@ class S3ApiBase(unittest.TestCase):
 
 class S3ApiBaseBoto3(S3ApiBase):
     def setUp(self):
+        if not tf.config.get('s3_access_key'):
+            raise SkipTest('no s3api user configured')
         if 's3api' not in tf.cluster_info:
-            raise tf.SkipTest('s3api middleware is not enabled')
+            raise SkipTest('s3api middleware is not enabled')
         try:
             self.conn = get_boto3_conn(
                 tf.config['s3_access_key'], tf.config['s3_secret_key'])
@@ -98,3 +112,29 @@ class S3ApiBaseBoto3(S3ApiBase):
 
     def tearDown(self):
         tear_down_s3(self.conn)
+
+
+def skip_boto2_sort_header_bug(m):
+    def wrapped(self, *args, **kwargs):
+        if os.environ.get('S3_USE_SIGV4') == "True":
+            # boto doesn't sort headers for v4 sigs properly; see
+            # https://github.com/boto/boto/pull/3032
+            # or https://github.com/boto/boto/pull/3176
+            # or https://github.com/boto/boto/pull/3751
+            # or https://github.com/boto/boto/pull/3824
+            self.skipTest('This stuff got the issue of boto<=2.x')
+        return m(self, *args, **kwargs)
+    return wrapped
+
+
+class SigV4Mixin(object):
+    @classmethod
+    def setUpClass(cls):
+        os.environ['S3_USE_SIGV4'] = "True"
+
+    @classmethod
+    def tearDownClass(cls):
+        del os.environ['S3_USE_SIGV4']
+
+    def setUp(self):
+        super(SigV4Mixin, self).setUp()
