@@ -18,6 +18,7 @@ from __future__ import print_function
 import errno
 import gc
 import json
+import mock
 import os
 from subprocess import Popen, PIPE
 import sys
@@ -345,6 +346,27 @@ class Body(object):
     next = __next__
 
 
+def exclude_nodes(nodes, *excludes):
+    """
+    Iterate over ``nodes`` yielding only those not in ``excludes``.
+
+    The index key of the node dicts is ignored when matching nodes against the
+    ``excludes`` nodes. Index is not a fundamental property of a node but a
+    variable annotation added by the Ring depending upon the partition for
+    which the nodes were generated.
+
+    :param nodes: an iterable of node dicts.
+    :param *excludes: one or more node dicts that should not be yielded.
+    :return: yields node dicts.
+    """
+    for node in nodes:
+        match_node = {k: mock.ANY if k == 'index' else v
+                      for k, v in node.items()}
+        if any(exclude == match_node for exclude in excludes):
+            continue
+        yield node
+
+
 class ProbeTest(unittest.TestCase):
     """
     Don't instantiate this directly, use a child class instead.
@@ -533,16 +555,18 @@ class ProbeTest(unittest.TestCase):
         for policy in ENABLED_POLICIES:
             for dev in policy.object_ring.devs:
                 all_obj_nodes[dev['device']] = dev
-        return all_obj_nodes.values()
+        return list(all_obj_nodes.values())
 
-    def gather_async_pendings(self, onodes):
+    def gather_async_pendings(self, onodes=None):
         """
         Returns a list of paths to async pending files found on given nodes.
 
-        :param onodes: a list of nodes.
+        :param onodes: a list of nodes. If None, check all object nodes.
         :return: a list of file paths.
         """
         async_pendings = []
+        if onodes is None:
+            onodes = self.get_all_object_nodes()
         for onode in onodes:
             device_dir = self.device_dir(onode)
             for ap_pol_dir in os.listdir(device_dir):
