@@ -29,7 +29,7 @@ from swift.common.storage_policy import StoragePolicy, POLICIES
 from swift.cli.info import (print_db_info_metadata, print_ring_locations,
                             print_info, print_obj_metadata, print_obj,
                             InfoSystemExit, print_item_locations,
-                            parse_get_node_args)
+                            parse_get_node_args, print_db_syncs)
 from swift.account.server import AccountController
 from swift.container.server import ContainerController
 from swift.container.backend import UNSHARDED, SHARDED
@@ -139,7 +139,8 @@ Metadata:
   UUID: abadf100d0ddba11
   X-Other-Something: boo
 No system metadata found in db file
-  User Metadata: {'x-account-meta-mydata': 'swift'}'''
+  User Metadata:
+    x-account-meta-mydata: swift'''
 
         self.assertEqual(out.getvalue().strip().split('\n'),
                          exp_out.split('\n'))
@@ -190,7 +191,8 @@ Metadata:
   UUID: abadf100d0ddba11
   X-Container-Bar: goo
   X-Container-Foo: bar
-  System Metadata: {'mydata': 'swift'}
+  System Metadata:
+    mydata: swift
 No user metadata found in db file
 Sharding Metadata:
   Type: root
@@ -232,7 +234,8 @@ Metadata:
   UUID: abadf100d0ddba11
   X-Other-Something: boo
 No system metadata found in db file
-  User Metadata: {'x-account-meta-mydata': 'swift'}'''
+  User Metadata:
+    x-account-meta-mydata: swift'''
 
         self.assertEqual(sorted(out.getvalue().strip().split('\n')),
                          sorted(exp_out.split('\n')))
@@ -663,6 +666,40 @@ Shard Ranges (3):
         self.assertIn(exp_cont_msg, out.getvalue())
         self.assertIn(exp_obj_msg, out.getvalue())
 
+    def test_print_db_syncs(self):
+        # first the empty case
+        for incoming in (True, False):
+            out = StringIO()
+            with mock.patch('sys.stdout', out):
+                print_db_syncs(incoming, [])
+            if incoming:
+                exp_heading = 'Incoming Syncs:'
+            else:
+                exp_heading = 'Outgoing Syncs:'
+            exp_heading += '\n  Sync Point\tRemote ID\tUpdated At'
+            self.assertIn(exp_heading, out.getvalue())
+
+        # now add some syncs
+        ts0 = utils.Timestamp(1)
+        ts1 = utils.Timestamp(2)
+        syncs = [{'sync_point': 0, 'remote_id': 'remote_0',
+                  'updated_at': str(int(ts0))},
+                 {'sync_point': 1, 'remote_id': 'remote_1',
+                  'updated_at': str(int(ts1))}]
+
+        template_output = """%s:\n  Sync Point\tRemote ID\tUpdated At
+  0         \tremote_0 \t%s (%s)
+  1         \tremote_1 \t%s (%s)
+"""
+        for incoming in (True, False):
+            out = StringIO()
+            with mock.patch('sys.stdout', out):
+                print_db_syncs(incoming, syncs)
+            output = template_output % (
+                'Incoming Syncs' if incoming else 'Outgoing Syncs',
+                ts0.isoformat, str(int(ts0)), ts1.isoformat, str(int(ts1)))
+            self.assertEqual(output, out.getvalue())
+
     def test_print_item_locations_account_container_object_dashed_ring(self):
         out = StringIO()
         account = 'account'
@@ -1219,7 +1256,7 @@ class TestPrintObjFullMeta(TestCliInfoBase):
             md = {'name': '/AUTH_admin/c/obj',
                   'Content-Type': 'application/octet-stream',
                   'ETag': 'd41d8cd98f00b204e9800998ecf8427e',
-                  'Content-Length': 0}
+                  'Content-Length': '0'}
             write_metadata(fp, md)
 
     def test_print_obj(self):
@@ -1256,7 +1293,7 @@ class TestPrintObjFullMeta(TestCliInfoBase):
             md = {'name': '/AUTH_admin/c/obj',
                   'Content-Type': 'application/octet-stream',
                   'ETag': 'd41d8cd98f00b204e9800998ecf8427e',
-                  'Content-Length': 0}
+                  'Content-Length': '0'}
             write_metadata(fp, md)
 
         object_ring = ring.Ring(self.testdir, ring_name='object-2')
@@ -1277,9 +1314,9 @@ class TestPrintObjFullMeta(TestCliInfoBase):
             os.chdir(cwd)
 
         exp_curl = (
-            'curl --path-as-is -g -I -XHEAD '
+            'curl -g -I -XHEAD '
             '"http://{host}:{port}/{device}/{part}/AUTH_admin/c/obj" '
-            '-H "X-Backend-Storage-Policy-Index: 2"').format(
+            '-H "X-Backend-Storage-Policy-Index: 2" --path-as-is').format(
                 host=node['ip'],
                 port=node['port'],
                 device=node['device'],
@@ -1297,7 +1334,7 @@ class TestPrintObjFullMeta(TestCliInfoBase):
             md = {'name': '/AUTH_admin/c/obj',
                   'Content-Type': 'application/octet-stream',
                   'ETag': 'd41d8cd98f00b204e9800998ecf8427e',
-                  'Content-Length': 0}
+                  'Content-Length': '0'}
             write_metadata(fp, md)
 
         object_ring = ring.Ring(self.testdir, ring_name='object-3')
@@ -1318,9 +1355,10 @@ class TestPrintObjFullMeta(TestCliInfoBase):
             os.chdir(cwd)
 
         exp_curl = (
-            'curl --path-as-is -g -I -XHEAD '
+            'curl -g -I -XHEAD '
             '"http://[{host}]:{port}'
-            '/{device}/{part}/AUTH_admin/c/obj" ').format(
+            '/{device}/{part}/AUTH_admin/c/obj" '
+            '-H "X-Backend-Storage-Policy-Index: 3" --path-as-is').format(
                 host=node['ip'],
                 port=node['port'],
                 device=node['device'],
@@ -1367,7 +1405,7 @@ class TestPrintObjFullMeta(TestCliInfoBase):
             md = {'name': '/AUTH_admin/c/obj',
                   'Content-Type': 'application/octet-stream',
                   'ETag': 'badetag',
-                  'Content-Length': 0}
+                  'Content-Length': '0'}
             write_metadata(fp, md)
 
         out = StringIO()
@@ -1702,5 +1740,5 @@ class TestPrintObjWeirdPath(TestPrintObjFullMeta):
             md = {'name': '/AUTH_admin/c/obj',
                   'Content-Type': 'application/octet-stream',
                   'ETag': 'd41d8cd98f00b204e9800998ecf8427e',
-                  'Content-Length': 0}
+                  'Content-Length': '0'}
             write_metadata(fp, md)
