@@ -12,8 +12,10 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import logging
 
 import mock
+import time
 import unittest
 from io import BytesIO
 from logging.handlers import SysLogHandler
@@ -21,22 +23,23 @@ from logging.handlers import SysLogHandler
 import six
 from six.moves.urllib.parse import unquote
 
-from swift.common.utils import get_logger, split_path, StatsdClient
+from swift.common.utils import get_logger, split_path
+from swift.common.statsd_client import StatsdClient
 from swift.common.middleware import proxy_logging
 from swift.common.registry import register_sensitive_header, \
     register_sensitive_param, get_sensitive_headers
-from swift.common.swob import Request, Response
+from swift.common.swob import Request, Response, HTTPServiceUnavailable
 from swift.common import constraints, registry
 from swift.common.storage_policy import StoragePolicy
-from test.debug_logger import debug_logger
+from test.debug_logger import debug_logger, FakeStatsdClient
 from test.unit import patch_policies
-from test.unit.common.middleware.helpers import FakeAppThatExcepts
+from test.unit.common.middleware.helpers import FakeAppThatExcepts, FakeSwift
 
 
 class FakeApp(object):
 
     def __init__(self, body=None, response_str='200 OK', policy_idx='0',
-                 chunked=False):
+                 chunked=False, environ_updates=None):
         if body is None:
             body = [b'FAKE APP']
         elif isinstance(body, six.binary_type):
@@ -46,6 +49,7 @@ class FakeApp(object):
         self.response_str = response_str
         self.policy_idx = policy_idx
         self.chunked = chunked
+        self.environ_updates = environ_updates or {}
 
     def __call__(self, env, start_response):
         try:
@@ -65,10 +69,11 @@ class FakeApp(object):
         if is_container_or_object_req and self.policy_idx is not None:
             headers.append(('X-Backend-Storage-Policy-Index',
                            str(self.policy_idx)))
-
         start_response(self.response_str, headers)
         while env['wsgi.input'].read(5):
             pass
+        # N.B. mw can set this anytime before the resp is finished
+        env.update(self.environ_updates)
         return self.body
 
 
@@ -114,6 +119,13 @@ def start_response(*args):
 
 @patch_policies([StoragePolicy(0, 'zero', False)])
 class TestProxyLogging(unittest.TestCase):
+    def setUp(self):
+        self.logger = debug_logger()
+        # really, this would come by way of base_prefix/tail_prefix in
+        # get_logger, ultimately tracing back to our hard-coded
+        # statsd_tail_prefix
+        self.logger.logger.statsd_client._prefix = 'proxy-server.'
+
     def _log_parts(self, app, should_be_empty=False):
         info_calls = app.access_logger.log_dict['info']
         if should_be_empty:
@@ -123,7 +135,7 @@ class TestProxyLogging(unittest.TestCase):
             return info_calls[0][0][0].split(' ')
 
     def assertTiming(self, exp_metric, app, exp_timing=None):
-        timing_calls = app.access_logger.log_dict['timing']
+        timing_calls = app.access_logger.statsd_client.calls['timing']
         found = False
         for timing_call in timing_calls:
             self.assertEqual({}, timing_call[1])
@@ -138,18 +150,117 @@ class TestProxyLogging(unittest.TestCase):
                 exp_metric, timing_calls))
 
     def assertNotTiming(self, not_exp_metric, app):
-        timing_calls = app.access_logger.log_dict['timing']
+        timing_calls = app.access_logger.statsd_client.calls['timing']
         for timing_call in timing_calls:
             self.assertNotEqual(not_exp_metric, timing_call[0][0])
 
     def assertUpdateStats(self, exp_metrics_and_values, app):
-        update_stats_calls = sorted(app.access_logger.log_dict['update_stats'])
+        update_stats_calls = sorted(
+            app.access_logger.statsd_client.calls['update_stats'])
         got_metrics_values_and_kwargs = [(usc[0][0], usc[0][1], usc[1])
                                          for usc in update_stats_calls]
         exp_metrics_values_and_kwargs = [(emv[0], emv[1], {})
                                          for emv in exp_metrics_and_values]
         self.assertEqual(got_metrics_values_and_kwargs,
                          exp_metrics_values_and_kwargs)
+        self.assertIs(self.logger, app.access_logger)
+        for metric, value in exp_metrics_and_values:
+            self.assertIn(
+                (('proxy-server.%s:%s|c' % (metric, value)).encode(),
+                 ('host', 8125)),
+                app.access_logger.statsd_client.sendto_calls)
+
+    def test_init_statsd_options_log_prefix(self):
+        conf = {
+            'log_headers': 'no',
+            'log_statsd_valid_http_methods': 'GET',
+            'log_facility': 'LOG_LOCAL7',
+            'log_name': 'bob',
+            'log_level': 'DEBUG',
+            'log_udp_host': 'example.com',
+            'log_udp_port': '3456',
+            'log_statsd_host': 'example.com',
+            'log_statsd_port': '1234',
+            'log_statsd_default_sample_rate': 10,
+            'log_statsd_sample_rate_factor': .04,
+            'log_statsd_metric_prefix': 'foo',
+        }
+        with mock.patch('swift.common.statsd_client.StatsdClient',
+                        FakeStatsdClient):
+            app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), conf)
+
+        self.assertFalse(app.log_hdrs)
+        self.assertEqual(['GET'], app.valid_methods)
+
+        log_adapter = app.access_logger
+        self.assertEqual('proxy-access', log_adapter.name)
+        self.assertEqual('bob', app.access_logger.server)
+        self.assertEqual(logging.DEBUG, log_adapter.logger.level)
+        self.assertEqual(('example.com', 3456),
+                         log_adapter.logger.handlers[0].address)
+        self.assertEqual(SysLogHandler.LOG_LOCAL7,
+                         log_adapter.logger.handlers[0].facility)
+
+        statsd_client = app.access_logger.logger.statsd_client
+        self.assertIsInstance(statsd_client, FakeStatsdClient)
+        with mock.patch.object(statsd_client, 'random', return_value=0):
+            statsd_client.increment('baz')
+        self.assertEqual(
+            [(b'foo.proxy-server.baz:1|c|@0.4', ('example.com', 1234))],
+            statsd_client.sendto_calls)
+
+    def test_init_statsd_options_access_log_prefix(self):
+        # verify that access_log_ prefix has precedence over log_
+        conf = {
+            'access_log_route': 'my-proxy-access',
+            'access_log_headers': 'yes',
+            'access_log_statsd_valid_http_methods': 'GET, HEAD',
+            'access_log_facility': 'LOG_LOCAL6',
+            'access_log_name': 'alice',
+            'access_log_level': 'WARN',
+            'access_log_udp_host': 'access.com',
+            'access_log_udp_port': '6789',
+            'log_headers': 'no',
+            'log_statsd_valid_http_methods': 'GET',
+            'log_facility': 'LOG_LOCAL7',
+            'log_name': 'bob',
+            'log_level': 'DEBUG',
+            'log_udp_host': 'example.com',
+            'log_udp_port': '3456',
+            'access_log_statsd_host': 'access.com',
+            'access_log_statsd_port': '5678',
+            'access_log_statsd_default_sample_rate': 20,
+            'access_log_statsd_sample_rate_factor': .03,
+            'access_log_statsd_metric_prefix': 'access_foo',
+            'log_statsd_host': 'example.com',
+            'log_statsd_port': '1234',
+            'log_statsd_default_sample_rate': 10,
+            'log_statsd_sample_rate_factor': .04,
+            'log_statsd_metric_prefix': 'foo',
+        }
+        with mock.patch('swift.common.statsd_client.StatsdClient',
+                        FakeStatsdClient):
+            app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), conf)
+
+        self.assertTrue(app.log_hdrs)
+        self.assertEqual(['GET', 'HEAD'], app.valid_methods)
+
+        log_adapter = app.access_logger
+        self.assertEqual('my-proxy-access', log_adapter.name)
+        self.assertEqual('alice', app.access_logger.server)
+        self.assertEqual(logging.WARN, log_adapter.logger.level)
+        self.assertEqual(('access.com', 6789),
+                         log_adapter.logger.handlers[0].address)
+        self.assertEqual(SysLogHandler.LOG_LOCAL6,
+                         log_adapter.logger.handlers[0].facility)
+
+        statsd_client = app.access_logger.logger.statsd_client
+        self.assertIsInstance(statsd_client, FakeStatsdClient)
+        with mock.patch.object(statsd_client, 'random', return_value=0):
+            statsd_client.increment('baz')
+        self.assertEqual(
+            [(b'access_foo.proxy-server.baz:1|c|@0.6', ('access.com', 5678))],
+            statsd_client.sendto_calls)
 
     def test_logger_statsd_prefix(self):
         app = proxy_logging.ProxyLoggingMiddleware(
@@ -172,26 +283,49 @@ class TestProxyLogging(unittest.TestCase):
                          app.access_logger.logger.statsd_client._prefix)
 
     def test_log_request_statsd_invalid_stats_types(self):
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = debug_logger()
-        for url in ['/', '/foo', '/foo/bar', '/v1']:
+        app = proxy_logging.ProxyLoggingMiddleware(
+            FakeApp(), {}, logger=self.logger)
+        for url in ['/', '/foo', '/foo/bar', '/v1', '/v1.0']:
+            self.logger.clear()
             req = Request.blank(url, environ={'REQUEST_METHOD': 'GET'})
             resp = app(req.environ, start_response)
             # get body
             b''.join(resp)
-            self.assertEqual([], app.access_logger.log_dict['timing'])
-            self.assertEqual([], app.access_logger.log_dict['update_stats'])
+            self.assertEqual(
+                [(('UNKNOWN.GET.200.first-byte.timing', mock.ANY), {}),
+                 (('UNKNOWN.GET.200.timing', mock.ANY), {})],
+                app.access_logger.statsd_client.calls['timing'])
+            self.assertEqual(
+                [(('UNKNOWN.GET.200.xfer', mock.ANY), {})],
+                app.access_logger.statsd_client.calls['update_stats'])
 
     def test_log_request_stat_type_bad(self):
-        for bad_path in ['', '/', '/bad', '/baddy/mc_badderson', '/v1',
-                         '/v1/']:
-            app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-            app.access_logger = debug_logger()
+        app = proxy_logging.ProxyLoggingMiddleware(
+            FakeApp(), {}, logger=self.logger)
+        for bad_path in [
+                '',
+                '/',
+                '/bad',
+                '/baddy/mc_badderson',
+                '/v1',
+                '/v1/',
+                '/v1.0',
+                '/v1.0/',
+                '/v1.0//',
+                '/v1.0//c',
+                '/v1.0/a//',
+                '/v1.0/a//o',
+        ]:
             req = Request.blank(bad_path, environ={'REQUEST_METHOD': 'GET'})
             now = 10000.0
             app.log_request(req, 123, 7, 13, now, now + 2.71828182846)
-            self.assertEqual([], app.access_logger.log_dict['timing'])
-            self.assertEqual([], app.access_logger.log_dict['update_stats'])
+            self.assertEqual(
+                [(('UNKNOWN.GET.123.timing', 2718.2818284600216), {})],
+                app.access_logger.statsd_client.calls['timing'])
+            self.assertEqual(
+                [(('UNKNOWN.GET.123.xfer', 20), {})],
+                app.access_logger.statsd_client.calls['update_stats'])
+            app.access_logger.clear()
 
     def test_log_request_stat_type_good(self):
         """
@@ -215,13 +349,24 @@ class TestProxyLogging(unittest.TestCase):
             '/v1/a/c/o/p': 'object',
             '/v1/a/c/o/p/': 'object',
             '/v1/a/c/o/p/p2': 'object',
+            '/v1.0/a': 'account',
+            '/v1.0/a/': 'account',
+            '/v1.0/a/c': 'container',
+            '/v1.0/a/c/': 'container',
+            '/v1.0/a/c/o': 'object',
+            '/v1.0/a/c/o/': 'object',
+            '/v1.0/a/c/o/p': 'object',
+            '/v1.0/a/c/o/p/': 'object',
+            '/v1.0/a/c/o/p/p2': 'object',
         }
         with mock.patch("time.time", stub_time):
             for path, exp_type in path_types.items():
                 # GET
+                self.logger.clear()
                 app = proxy_logging.ProxyLoggingMiddleware(
-                    FakeApp(body=b'7654321', response_str='321 Fubar'), {})
-                app.access_logger = debug_logger()
+                    FakeApp(body=b'7654321', response_str='321 Fubar'),
+                    {},
+                    logger=self.logger)
                 req = Request.blank(path, environ={
                     'REQUEST_METHOD': 'GET',
                     'wsgi.input': BytesIO(b'4321')})
@@ -231,15 +376,20 @@ class TestProxyLogging(unittest.TestCase):
                 self.assertEqual(b'7654321', b''.join(iter_response))
                 self.assertTiming('%s.GET.321.timing' % exp_type, app,
                                   exp_timing=2.71828182846 * 1000)
+                self.assertTiming('%s.GET.321.first-byte.timing'
+                                  % exp_type, app, exp_timing=0.5 * 1000)
                 if exp_type == 'object':
                     # Object operations also return stats by policy
                     # In this case, the value needs to match the timing for GET
                     self.assertTiming('%s.policy.0.GET.321.timing' % exp_type,
                                       app, exp_timing=2.71828182846 * 1000)
+                    self.assertTiming(
+                        '%s.policy.0.GET.321.first-byte.timing'
+                        % exp_type, app, exp_timing=0.5 * 1000)
                     self.assertUpdateStats([('%s.GET.321.xfer' % exp_type,
                                              4 + 7),
                                             ('object.policy.0.GET.321.xfer',
-                                            4 + 7)],
+                                             4 + 7)],
                                            app)
                 else:
                     self.assertUpdateStats([('%s.GET.321.xfer' % exp_type,
@@ -249,10 +399,10 @@ class TestProxyLogging(unittest.TestCase):
                 # GET Repeat the test above, but with a non-existent policy
                 # Do this only for object types
                 if exp_type == 'object':
+                    self.logger.clear()
                     app = proxy_logging.ProxyLoggingMiddleware(
                         FakeApp(body=b'7654321', response_str='321 Fubar',
-                                policy_idx='-1'), {})
-                    app.access_logger = debug_logger()
+                                policy_idx='-1'), {}, logger=self.logger)
                     req = Request.blank(path, environ={
                         'REQUEST_METHOD': 'GET',
                         'wsgi.input': BytesIO(b'4321')})
@@ -278,16 +428,18 @@ class TestProxyLogging(unittest.TestCase):
                 stub_times = [18.0, 20.71828182846]
                 iter_response = app(req.environ, lambda *_: None)
                 self.assertEqual(b'7654321', b''.join(iter_response))
-                self.assertEqual([], app.access_logger.log_dict['timing'])
-                self.assertEqual([],
-                                 app.access_logger.log_dict['timing_since'])
-                self.assertEqual([],
-                                 app.access_logger.log_dict['update_stats'])
+                self.assertEqual(
+                    [], app.access_logger.statsd_client.calls['timing'])
+                self.assertEqual(
+                    [], app.access_logger.statsd_client.calls['timing_since'])
+                self.assertEqual(
+                    [], app.access_logger.statsd_client.calls['update_stats'])
 
                 # PUT (no first-byte timing!)
+                self.logger.clear()
                 app = proxy_logging.ProxyLoggingMiddleware(
-                    FakeApp(body=b'87654321', response_str='314 PiTown'), {})
-                app.access_logger = debug_logger()
+                    FakeApp(body=b'87654321', response_str='314 PiTown'), {},
+                    logger=self.logger)
                 req = Request.blank(path, environ={
                     'REQUEST_METHOD': 'PUT',
                     'wsgi.input': BytesIO(b'654321')})
@@ -317,10 +469,10 @@ class TestProxyLogging(unittest.TestCase):
                 # PUT Repeat the test above, but with a non-existent policy
                 # Do this only for object types
                 if exp_type == 'object':
+                    self.logger.clear()
                     app = proxy_logging.ProxyLoggingMiddleware(
                         FakeApp(body=b'87654321', response_str='314 PiTown',
-                                policy_idx='-1'), {})
-                    app.access_logger = debug_logger()
+                                policy_idx='-1'), {}, logger=self.logger)
                     req = Request.blank(path, environ={
                         'REQUEST_METHOD': 'PUT',
                         'wsgi.input': BytesIO(b'654321')})
@@ -335,8 +487,8 @@ class TestProxyLogging(unittest.TestCase):
                     self.assertNotTiming(
                         '%s.PUT.314.first-byte.timing' % exp_type, app)
                     # No results returned for the non-existent policy
-                    self.assertUpdateStats([('object.PUT.314.xfer', 6 + 8)],
-                                           app)
+                    self.assertUpdateStats(
+                        [('object.PUT.314.xfer', 6 + 8)], app)
 
     def test_log_request_stat_method_filtering_default(self):
         method_map = {
@@ -353,8 +505,9 @@ class TestProxyLogging(unittest.TestCase):
             'OPTIONS': 'OPTIONS',
         }
         for method, exp_method in method_map.items():
-            app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-            app.access_logger = debug_logger()
+            self.logger.clear()
+            app = proxy_logging.ProxyLoggingMiddleware(
+                FakeApp(), {}, logger=self.logger)
             req = Request.blank('/v1/a/', environ={'REQUEST_METHOD': method})
             now = 10000.0
             app.log_request(req, 299, 11, 3, now, now + 1.17)
@@ -377,10 +530,10 @@ class TestProxyLogging(unittest.TestCase):
         for conf_key in ['access_log_statsd_valid_http_methods',
                          'log_statsd_valid_http_methods']:
             for method, exp_method in method_map.items():
+                self.logger.clear()
                 app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
                     conf_key: 'SPECIAL,  GET,PUT ',  # crazy spaces ok
-                })
-                app.access_logger = debug_logger()
+                }, logger=self.logger)
                 req = Request.blank('/v1/a/c',
                                     environ={'REQUEST_METHOD': method})
                 now = 10000.0
@@ -404,6 +557,112 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(resp_body, b'FAKE APP')
         self.assertEqual(log_parts[11], str(len(resp_body)))
 
+    def test_object_error(self):
+        swift = FakeSwift()
+        self.logger = debug_logger()
+        app = proxy_logging.ProxyLoggingMiddleware(swift, {},
+                                                   logger=self.logger)
+        swift.register('GET', '/v1/a/c/o', HTTPServiceUnavailable, {}, None)
+        req = Request.blank('/v1/a/c/o')
+        start = time.time()
+        ttfb = start + 0.2
+        end = ttfb + 0.5
+        with mock.patch("swift.common.middleware.proxy_logging.time.time",
+                        side_effect=[start, ttfb, end]):
+            resp = req.get_response(app)
+            self.assertEqual(503, resp.status_int)
+            # we have to consume the resp body to trigger logging
+            self.assertIn(b'Service Unavailable', resp.body)
+            log_parts = self._log_parts(app)
+        self.assertEqual(log_parts[3], 'GET')
+        self.assertEqual(log_parts[4], '/v1/a/c/o')
+        self.assertEqual(log_parts[5], 'HTTP/1.0')
+        self.assertEqual(log_parts[6], '503')
+        # we can also expect error metrics
+        self.assertTiming('object.GET.503.timing', app,
+                          exp_timing=700.0)
+        self.assertTiming('object.GET.503.first-byte.timing', app,
+                          exp_timing=200.0)
+
+    def test_basic_error(self):
+        swift = FakeSwift()
+        self.logger = debug_logger()
+        app = proxy_logging.ProxyLoggingMiddleware(swift, {},
+                                                   logger=self.logger)
+        swift.register('GET', '/path', HTTPServiceUnavailable, {}, None)
+        req = Request.blank('/path')
+        start = time.time()
+        ttfb = start + 0.2
+        end = ttfb + 0.5
+        with mock.patch("swift.common.middleware.proxy_logging.time.time",
+                        side_effect=[start, ttfb, end]):
+            resp = req.get_response(app)
+            self.assertEqual(503, resp.status_int)
+            # we have to consume the resp body to trigger logging
+            self.assertIn(b'Service Unavailable', resp.body)
+            log_parts = self._log_parts(app)
+        self.assertEqual(log_parts[3], 'GET')
+        self.assertEqual(log_parts[4], '/path')
+        self.assertEqual(log_parts[5], 'HTTP/1.0')
+        self.assertEqual(log_parts[6], '503')
+        # we can also expect error metrics
+        self.assertTiming('UNKNOWN.GET.503.timing', app,
+                          exp_timing=700.0)
+        self.assertTiming('UNKNOWN.GET.503.first-byte.timing', app,
+                          exp_timing=200.0)
+
+    def test_middleware_exception(self):
+        self.logger = debug_logger()
+        app = proxy_logging.ProxyLoggingMiddleware(
+            FakeAppThatExcepts(), {}, logger=self.logger)
+        req = Request.blank('/path')
+        start = time.time()
+        ttfb = start + 0.2
+        with mock.patch("swift.common.middleware.proxy_logging.time.time",
+                        side_effect=[start, ttfb]), \
+                self.assertRaises(Exception):
+            req.get_response(app)
+        log_parts = self._log_parts(app)
+        self.assertEqual(log_parts[3], 'GET')
+        self.assertEqual(log_parts[4], '/path')
+        self.assertEqual(log_parts[5], 'HTTP/1.0')
+        self.assertEqual(log_parts[6], '500')
+        # we can also expect error metrics
+        self.assertTiming('UNKNOWN.GET.500.timing', app,
+                          exp_timing=200.0)
+
+    def test_middleware_error(self):
+        class ErrorFakeApp(object):
+
+            def __call__(self, env, start_response):
+                env['swift.source'] = 'FA'
+                resp = HTTPServiceUnavailable()
+                return resp(env, start_response)
+
+        self.logger = debug_logger()
+        app = proxy_logging.ProxyLoggingMiddleware(ErrorFakeApp(), {},
+                                                   logger=self.logger)
+        req = Request.blank('/path')
+        start = time.time()
+        ttfb = start + 0.2
+        end = ttfb + 0.5
+        with mock.patch("swift.common.middleware.proxy_logging.time.time",
+                        side_effect=[start, ttfb, end]):
+            resp = req.get_response(app)
+            self.assertEqual(503, resp.status_int)
+            # we have to consume the resp body to trigger logging
+            self.assertIn(b'Service Unavailable', resp.body)
+            log_parts = self._log_parts(app)
+        self.assertEqual(log_parts[3], 'GET')
+        self.assertEqual(log_parts[4], '/path')
+        self.assertEqual(log_parts[5], 'HTTP/1.0')
+        self.assertEqual(log_parts[6], '503')
+        # we can also expect error metrics
+        self.assertTiming('FA.GET.503.timing', app,
+                          exp_timing=700.0)
+        self.assertTiming('FA.GET.503.first-byte.timing', app,
+                          exp_timing=200.0)
+
     def test_basic_req_second_time(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
         app.access_logger = debug_logger()
@@ -452,8 +711,8 @@ class TestProxyLogging(unittest.TestCase):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
             'log_msg_template': (
                 '{protocol} {path} {method} '
-                '{account} {container} {object}')})
-        app.access_logger = debug_logger()
+                '{account} {container} {object}')
+        }, logger=self.logger)
         req = Request.blank('/bucket/path/to/key', environ={
             'REQUEST_METHOD': 'GET',
             # This would actually get set in the app, but w/e
@@ -493,8 +752,7 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_multi_segment_resp(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(
-            [b'some', b'chunks', b'of data']), {})
-        app.access_logger = debug_logger()
+            [b'some', b'chunks', b'of data']), {}, logger=self.logger)
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
                                           'swift.source': 'SOS'})
         resp = app(req.environ, start_response)
@@ -506,20 +764,22 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(log_parts[6], '200')
         self.assertEqual(resp_body, b'somechunksof data')
         self.assertEqual(log_parts[11], str(len(resp_body)))
-        self.assertUpdateStats([('SOS.GET.200.xfer', len(resp_body))], app)
+        self.assertUpdateStats([('SOS.GET.200.xfer', len(resp_body))],
+                               app)
 
     def test_log_headers(self):
         for conf_key in ['access_log_headers', 'log_headers']:
+            self.logger.clear()
             app = proxy_logging.ProxyLoggingMiddleware(FakeApp(),
-                                                       {conf_key: 'yes'})
-            app.access_logger = debug_logger()
+                                                       {conf_key: 'yes'},
+                                                       logger=self.logger)
             req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
             resp = app(req.environ, start_response)
             # exhaust generator
             [x for x in resp]
             log_parts = self._log_parts(app)
             headers = unquote(log_parts[14]).split('\n')
-            self.assertTrue('Host: localhost:80' in headers)
+            self.assertIn('Host: localhost:80', headers)
 
     def test_access_log_headers_only(self):
         app = proxy_logging.ProxyLoggingMiddleware(
@@ -544,8 +804,8 @@ class TestProxyLogging(unittest.TestCase):
     def test_upload_size(self):
         # Using default policy
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(),
-                                                   {'log_headers': 'yes'})
-        app.access_logger = debug_logger()
+                                                   {'log_headers': 'yes'},
+                                                   logger=self.logger)
         req = Request.blank(
             '/v1/a/c/o/foo',
             environ={'REQUEST_METHOD': 'PUT',
@@ -563,9 +823,10 @@ class TestProxyLogging(unittest.TestCase):
                                app)
 
         # Using a non-existent policy
+        self.logger.clear()
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(policy_idx='-1'),
-                                                   {'log_headers': 'yes'})
-        app.access_logger = debug_logger()
+                                                   {'log_headers': 'yes'},
+                                                   logger=self.logger)
         req = Request.blank(
             '/v1/a/c/o/foo',
             environ={'REQUEST_METHOD': 'PUT',
@@ -582,8 +843,8 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_upload_size_no_policy(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(policy_idx=None),
-                                                   {'log_headers': 'yes'})
-        app.access_logger = debug_logger()
+                                                   {'log_headers': 'yes'},
+                                                   logger=self.logger)
         req = Request.blank(
             '/v1/a/c/o/foo',
             environ={'REQUEST_METHOD': 'PUT',
@@ -600,8 +861,8 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_upload_line(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeAppReadline(),
-                                                   {'log_headers': 'yes'})
-        app.access_logger = debug_logger()
+                                                   {'log_headers': 'yes'},
+                                                   logger=self.logger)
         req = Request.blank(
             '/v1/a/c',
             environ={'REQUEST_METHOD': 'POST',
@@ -789,6 +1050,193 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(log_parts[6], '499')
         self.assertEqual(log_parts[10], '-')  # read length
 
+    def test_environ_has_proxy_logging_status(self):
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        def do_test(environ_updates):
+            fake_app = FakeApp(body=[b'Slow Down'],
+                               response_str='503 Slow Down',
+                               environ_updates=environ_updates)
+            app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+            app.access_logger = debug_logger()
+            req = Request.blank('/v1/a/c')
+            captured_start_resp = mock.MagicMock()
+            try:
+                resp = app(req.environ, captured_start_resp)
+                b''.join(resp)  # read body
+            except IOError:
+                pass
+            captured_start_resp.assert_called_once_with(
+                '503 Slow Down', mock.ANY, None)
+            return self._log_parts(app)
+
+        # control case, logged status == wire status
+        environ_updates = {}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '503', '503'])
+
+        # logged status is forced to other value
+        environ_updates = {'swift.proxy_logging_status': 429}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '429', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': '429'}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '429', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': None}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '-', '503'])
+
+        # middleware should use an int like the docs tell them too, but we
+        # won't like ... "blow up" or anything
+        environ_updates = {'swift.proxy_logging_status': ''}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '-', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': True}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', 'True', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': False}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '-', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': 'parsing ok'}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', 'parsing%20ok', '503'])
+
+    def test_body_iter_updates_environ_proxy_logging_status(self):
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        def do_test(req, body_iter, updated_status):
+            fake_app = FakeApp(body=body_iter,
+                               response_str='205 Weird')
+            app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+            app.access_logger = debug_logger()
+            captured_start_resp = mock.MagicMock()
+            try:
+                resp = app(req.environ, captured_start_resp)
+                b''.join(resp)  # read body
+            except IOError:
+                pass
+            captured_start_resp.assert_called_once_with(
+                '205 Weird', mock.ANY, None)
+            self.assertEqual(self._log_parts(app),
+                             ['GET', '/v1/a/c', updated_status, '205'])
+
+        # sanity
+        req = Request.blank('/v1/a/c')
+        do_test(req, [b'normal', b'chunks'], '205')
+
+        def update_in_middle_chunk_gen():
+            yield b'foo'
+            yield b'bar'
+            req.environ['swift.proxy_logging_status'] = 209
+            yield b'baz'
+
+        req = Request.blank('/v1/a/c')
+        do_test(req, update_in_middle_chunk_gen(), '209')
+
+        def update_in_finally_chunk_gen():
+            try:
+                for i in range(3):
+                    yield ('foo%s' % i).encode()
+            finally:
+                req.environ['swift.proxy_logging_status'] = 210
+
+        req = Request.blank('/v1/a/c')
+        do_test(req, update_in_finally_chunk_gen(), '210')
+
+    def test_environ_has_proxy_logging_status_unread_body(self):
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        def do_test(environ_updates):
+            fake_app = FakeApp(body=[b'Slow Down'],
+                               response_str='503 Slow Down',
+                               environ_updates=environ_updates)
+            app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+            app.access_logger = debug_logger()
+            req = Request.blank('/v1/a/c')
+            captured_start_resp = mock.MagicMock()
+            resp = app(req.environ, captured_start_resp)
+            # read first chunk
+            next(resp)
+            resp.close()  # raise a GeneratorExit in middleware app_iter loop
+            captured_start_resp.assert_called_once_with(
+                '503 Slow Down', mock.ANY, None)
+            return self._log_parts(app)
+
+        # control case, logged status is 499
+        environ_updates = {}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '499', '503'])
+
+        # logged status is forced to 499 despite swift.proxy_logging_status
+        environ_updates = {'swift.proxy_logging_status': '429'}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '499', '503'])
+
+    def test_environ_has_proxy_logging_status_and_app_explodes(self):
+        # verify exception overrides proxy_logging_status
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        class ExplodingFakeApp(object):
+
+            def __call__(self, env, start_response):
+                # this is going to be so great!
+                env['swift.proxy_logging_status'] = '456'
+                start_response('568 Bespoke', [('X-Special', 'fun')])
+                raise Exception('oops!')
+
+        fake_app = ExplodingFakeApp()
+        app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+        app.access_logger = debug_logger()
+        req = Request.blank('/v1/a/c')
+        captured_start_resp = mock.MagicMock()
+        with self.assertRaises(Exception) as cm:
+            app(req.environ, captured_start_resp)
+        captured_start_resp.assert_not_called()
+        self.assertEqual('oops!', str(cm.exception))
+        self.assertEqual(self._log_parts(app),
+                         ['GET', '/v1/a/c', '500', '500'])
+
+    def test_environ_has_proxy_logging_status_and_body_explodes(self):
+        # verify exception overrides proxy_logging_status
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        def exploding_body():
+            yield 'some'
+            yield 'stuff'
+            raise Exception('oops!')
+
+        class ExplodingFakeApp(object):
+
+            def __call__(self, env, start_response):
+                # this is going to be so great!
+                env['swift.proxy_logging_status'] = '456'
+                start_response('568 Bespoke', [('X-Special', 'fun')])
+                return exploding_body()
+
+        fake_app = ExplodingFakeApp()
+        app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+        app.access_logger = debug_logger()
+        req = Request.blank('/v1/a/c')
+        captured_start_resp = mock.MagicMock()
+        app_iter = app(req.environ, captured_start_resp)
+        with self.assertRaises(Exception) as cm:
+            b''.join(app_iter)
+        captured_start_resp.assert_called_once_with(
+            '568 Bespoke', [('X-Special', 'fun')], None)
+        self.assertEqual('oops!', str(cm.exception))
+        self.assertEqual(self._log_parts(app),
+                         ['GET', '/v1/a/c', '500', '568'])
+
     def test_app_exception(self):
         app = proxy_logging.ProxyLoggingMiddleware(
             FakeAppThatExcepts(), {})
